#include "Cut.h"
#include "Copy.h"
#include "Earse.h"
#include <fcntl.h>
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include "DspLib.h"
#include "Errors.h"
#include "Select.h"
#include "Fct_Sys.h"
#include "Convert.h"
#include "DebugLib.h"
#include "8bM_16bS.h"
#include "WavePlay.h"
#include "Set_Rate.h"
#include <sys/stat.h>
#include "ConfigLib.h"
#include "Constants.h"
#include <sys/types.h>
#include "WaveHeader.h"
#include "Swap_Chanels.h"
#include "sound_sample.h"
#include "Charge_Sound.h"
#include "Charge_Filter.h"
#include "Parse_Filter_Conf.h"
#include "Parse_Loader_Conf.h"

typedef struct s_playmode
{
  int	chanels;      /* 0 => gauche, 1 => droit, autre => gauche & droit */
}	t_playmode;


void	affiche_per100_b(nb_unit, length)
off_t	nb_unit;
off_t	length;     
{
  float	per100;
 
  per100 = ((double) nb_unit) / (double) (length);
#ifdef TEXT_MSG
  printf("%1.1f%%, pos = %d\n", per100 * 100, 2 * (int) (nb_unit * sizeof(short int)));
#endif
}

/*
** set_buffer: modifie le buffer en fonction des option donnees
*/
void		set_buffer_b(buffer, nb, playmode)
short int	*buffer;
size_t		nb;
t_playmode	*playmode;
{
  long int	cpt;
  
  if ((playmode->chanels == 0) || (playmode->chanels == 1))
    for (cpt = 0; cpt < nb; cpt++)
      {
	if (playmode->chanels + (cpt % 2) == 1)
	  buffer[cpt] *= 0;
	else
	  buffer[cpt] *= 1;
      }
}

/*
** init_play_file: initialise le player de fichier temporaire
*/
int	init_play_file_b(curr_pos, file, pos, length)
off_t	*curr_pos;
int	file;
off_t	pos;
off_t	*length;
{
  *curr_pos = lseek(file, pos * 4, SEEK_SET);
  if (*curr_pos == -1)
    {
#ifdef TEXT_MSG
      printf("%s\n", BAD_SEEK);
#endif
      return (-1);
    }
  *curr_pos /= 4;
  if (*length == 0)        
    *length = (give_file_size(file) / 4) - pos; 
  return (0);
}

/*
** play_file: lit les donnees et les joue sur le dsp
*/
void		play_file_b(file, nb, dsp_file, playmode)
int		file;
size_t		nb;
int		dsp_file;
t_playmode	playmode;
{
  //size_t	cpt;
  short int	buffer[BUFFER_SIZE * 2]; /* this buffer contain BUFFER_SIZE sound units */

  read(file, &buffer, 2 * nb * sizeof(short  int));
  //for (cpt = 0; cpt < 20; cpt += 2)
  //printf("cpt = %d, nb = %d, %d\t\t%d\n", cpt, nb, buffer[cpt], buffer[cpt + 1]);
  set_buffer_b(buffer, 2 * nb, playmode);
  write(dsp_file, &buffer, 2 * nb * sizeof(short int));
}

/*
** play_temp_file: joue un fichier temporaire de la position donnee (en unite de son)
**		   sur la longeur donne (en unites de son) sur le dsp donne et dans le
**		   mode donne
*/
int		play_temp_file_b(file, dsp_file, pos, _length, playmode)
int		file;
int		dsp_file;
off_t		pos;
off_t		_length;
t_playmode	*playmode;
{
  size_t	nb_unit;
  size_t	curr_pos;
  size_t	length;
  size_t	nb;

  length = _length;
  if (init_play_file_b(&curr_pos, file, pos, &_length) == -1)
    return (-1);
  nb_unit = 0;
  while (nb_unit < _length)
    {
      if (nb_unit + BUFFER_SIZE <= _length)
	nb = BUFFER_SIZE;
      else
	nb = _length - nb_unit;
      nb_unit += nb;
      play_file_b(file, nb, dsp_file, playmode);
      affiche_per100_b(nb_unit, _length);
    }
  return (0);
}


/*
****************************************************************************************
**                FONCTIONS DE  MINIPROG                                              **
****************************************************************************************
*/

/*
** affiche_menu: affiche le menu de MiniProg
*/
void affiche_menu()
{
  printf("1) copier\n");
  printf("2) inverser les canaux\n");
  printf("3) effacer\n");
  printf("4) couper\n");
  printf("c) charger un fichier WAV\n");
  printf("f) lancer un filtre\n");
  printf("i) lancer l'inversion de canaux\n");
  printf("j) jouer le fichier temporaire\n");
  printf("q) quitter\n");
}

/*
** init_prog: initialise le MiniProg (alloue de la memoire 
**            pour wav_path et initialise le dsp)
*/
void init_prog(wav_path, dsp_file, rate)
     char	**wav_path;
     t_dsp	**dsp_file;
     int	rate;
{
  printf("      *****                                      \n");
  printf("     *     *                           **        \n");
  printf("    *                                 * *        \n");
  printf("    *    ****  ** ** **   *  *   ***  * *        \n");
  printf("    *      *     *  *  *  *  *   *  *  *         \n");
  printf("     ******      *  *  * * ** * *   * * *        \n");
  printf("                                        *        \n");
  printf("                                      * *        \n");
  printf("                                       *         \n");
  *wav_path = (char *) malloc (50);
  printf ("\n\n\n   Ah que coucou!\n       Bienvenu a bord de MiniProg\n\n");
  *dsp_file = (t_dsp *) malloc (sizeof (t_dsp));
  //*dsp_file = open_dsp_for_play(DSP_DEVICE, 16, 2, rate);
}

/*
** close_all: libere toute la memoire allouee et ferme tous les fichiers ouverts
*/
void	close_all(temp_gmurf, dsp_file)
     t_tempgmurf	*temp_gmurf;
     t_dsp		*dsp_file;
{
  if (temp_gmurf != NULL)
    {
      close(temp_gmurf->file); 
      unlink(temp_gmurf->tempname);
      free(temp_gmurf);
    }
  close_dsp(dsp_file);
}

void		swap(nb1, nb2)
short int	*nb1;
short int	*nb2;
{
  short int	temp;
  
  temp = *nb1;
  *nb1 = *nb2;
  *nb2 = temp;
}

void		invert(buf, nb_word)
short int	*buf;
int		nb_word;
{
  int	cpt;
  int	half_buf;
  
  half_buf = nb_word / 2;
  for(cpt = 0; cpt < half_buf; cpt += 2)
    {
      swap(&(buf[cpt]),     &(buf[nb_word - cpt - 2]));
      swap(&(buf[cpt + 1]), &(buf[nb_word - cpt - 1]));
    }
}

void		get_other_direction(buf1, buf2, chanels2change, nb_word2read)
short int	*buf1;
short int       *buf2;
int		chanels2change;
long int	nb_word2read;
{      
  long	int	compt;

  if (chanels2change == 1)
    for (compt = 0; compt < nb_word2read; compt++)
      if (compt % 2 != 0)
	buf1[compt] = buf2[compt];
  if (chanels2change == 2)
    for (compt = 0; compt < nb_word2read; compt++)
      if (compt % 2 == 0)
	buf1[compt] = buf2[compt];
}
void		invert_chanels(temp_gmurf, chanels2change)
t_tempgmurf	*temp_gmurf;
int		chanels2change;
{
  int		fd;
  int		cpt;
  long  int	nb_word2read;
  long	int	pos2seek;
  long	int	nb_word_read;
  long  int	temp_size_in_word;
  short int	buf1[BUFFER_SIZE];
  short int	buf2[BUFFER_SIZE];

  fd = open(TEMP_FILE, O_RDWR | O_CREAT | O_TRUNC, 0644);
  cpt = 1;
  nb_word_read = 0;
  temp_size_in_word = temp_gmurf->temp_size / sizeof(short int);
  while (nb_word_read < temp_size_in_word)
    {
      pos2seek = temp_gmurf->temp_size - (sizeof(buf1) * (unsigned long) cpt);
      //pos2seek = temp_gmurf->temp_size - (BUFFER_SIZE * sizeof(short int)) - nb_word_read;
      if (pos2seek < 0) 
	pos2seek = 0;
      lseek(temp_gmurf->file, pos2seek, SEEK_SET);
      if (nb_word_read + BUFFER_SIZE  <= temp_size_in_word)
	  nb_word2read = BUFFER_SIZE;
      else
	nb_word2read = temp_size_in_word - nb_word_read;
      read(temp_gmurf->file, buf1, nb_word2read * sizeof(short int));
      invert(buf1, nb_word2read);
      lseek(temp_gmurf->file, nb_word_read * sizeof(short int), SEEK_SET);
      read(temp_gmurf->file, buf2, nb_word2read * sizeof(short int));
      get_other_direction(buf1, buf2, chanels2change, nb_word2read);
      write(fd, buf1, nb_word2read * sizeof(short int));
      nb_word_read += nb_word2read;
      cpt++;
    }
  close(temp_gmurf->file);
  close(fd);
  unlink(temp_gmurf->tempname);
  rename(TEMP_FILE, temp_gmurf->tempname);
  temp_gmurf->file = open(temp_gmurf->tempname, O_RDWR /*| O_APPEND*/);
}

/*
** run_filter: lance un filtre
*/
int		run_filter(temp_gmurf, filters, gmurfrc)
t_tempgmurf	*temp_gmurf;
t_filters	*filters;
t_gmurfrc	*gmurfrc;
{
  int		cpt;
  int		choix;
  t_filter	*filter_ptr;
  t_category	*category_ptr;
  t_filter_data	*filter_data;
  
  filter_data = (t_filter_data *) malloc(sizeof(t_filter_data));
  filter_data->rate   = 44100;
  filter_data->begin  = 0;
  filter_data->length = 0;
  category_ptr        = filters->category;
  cpt = 1;
  while (cpt <= filters->nb_category)
    {
      printf("%d) %s\n", cpt, category_ptr->name);
      category_ptr = category_ptr->next;
      cpt++;
    }
  do
    {
      printf(" votre choix:");
      scanf("%d", &choix);
    } while ((choix >= cpt) || (choix <= 0));
  category_ptr = filters->category;
  cpt = 1;
  while (cpt++ < choix)
    category_ptr = category_ptr->next;
  printf("%s\n\n", category_ptr->name);
  cpt = 1;
  filter_ptr = category_ptr->filter;
  while (cpt <= category_ptr->nb_filters)
    {
      printf("%d) %s\n", cpt, filter_ptr->name);
      filter_ptr = filter_ptr->next;
      cpt++;
    }
  do
    {
      printf(" votre choix:");
      scanf("%d", &choix);
    } while ((choix >= cpt) || (choix <= 0));  
  cpt = 1;
  filter_ptr = category_ptr->filter;
  while (cpt++ < choix)
       filter_ptr = filter_ptr->next;
  printf("%s\n\n", filter_ptr->name);
  exec_filter(filter_ptr->path, filter_data, temp_gmurf);
  return (0);
}


int		translate(temp_gmurf, chanel, value)
t_tempgmurf	*temp_gmurf;
int		chanel;
int		value;
{
  int			fd;
  off_t			cpt;
  off_t			compt;
  off_t			length;		   /* length of the target file */
  off_t			nb2write;	   /* number of byte to write on the target in 1 loop */
  off_t			pos_left;	   /* position in the left  chanel */
  off_t			pos_right;	   /* position in the right chanel */
  off_t			value_left;	   /* value of the left  chanel's translation */
  off_t			value_right;	   /* value of the right chanel's translation */
  off_t			length_left;	   /* length of the left  chanel */
  off_t			length_right;	   /* length of the right chanel */
  t_sound_sample	buf_src[BUFFER_SIZE];
  t_sound_sample	buf_trg[BUFFER_SIZE];

  rename(temp_gmurf->tempname, UNDO_FILE);
  fd = temp_gmurf->file;
  temp_gmurf->file = open(temp_gmurf->tempname, O_CREAT | O_TRUNC | O_APPEND | O_RDWR, 0644);
  length = temp_gmurf->temp_size / sizeof(t_sound_sample);
  if ((value > 0) || (chanel == 2))
    length += value;
  compt     = 0;
  pos_left	= chanel != 1 ? 0 : value; 
  value_left	= chanel != 0 ? 0 : value; 
  value_right	= chanel != 1 ? 0 : value;
  length_left	= temp_gmurf->temp_size / sizeof(t_sound_sample) + value_left;
  length_right	= temp_gmurf->temp_size / sizeof(t_sound_sample) + value_right;
  while (compt < length)
    {
      nb2write = compt + BUFFER_SIZE < length ? BUFFER_SIZE : nb2write - compt;
      for (cpt = 0; cpt < nb2write; cpt++)
	buf_trg[cpt].left = buf_trg[cpt].right = 0;

      lseek(fd, pos_left * sizeof(t_tempgmurf), SEEK_SET);
      read(fd, buf_src, nb2write * sizeof(t_tempgmruf));
      for (cpt = 0; cpt < nb2write; cpt++)
	{
	  if ((pos_left + cpt >= value_left) && (pos_left + cpt <= length_left))
	    {
	      
	      pos_left++;
	    }
	  else
	    buf_src[cpt].left = 0;
	}
      //if ((compt + nb2write >= value_left) && (compt <= value_left + length_left))
      //{
      //  lseek(fd, compt - value_left, SEEK_SET);
	  //read(fd, buf_src, );
      //}
      write(temp_gmurf->file, buf_trg, nb2write * sizeof(t_sound_sample));
      compt += nb2write;
    }
  return (1);
}

/*
****************************************************************************************
**                FONCTION MAIN                                                       **
****************************************************************************************
*/
int main(argc, argv)
     int	argc;
     char	*argv[];
{
  int			is_wav = 1;
  char			choix = ' ';
  char			*wav_path;
  t_dsp			*dsp_file;
  t_select		select;
  t_filters		*filters;
  t_gmurfrc		*gmurfrc;
  t_tempgmurf		*temp_gmurf;
  //t_sound_sample	sound_sample;

  select.begin		= 0;
  select.length		= 4410;
  select.reverse	= 0;
  select.is_selected	= 1;
  select.chanel		= 1;

  test_sound_file_loader();
  if ((filters = get_filter_path(FILTER_LOADER)) == NULL)
    exit(-1);
  printf("ici passe\n");
  if (mkdir(TMP_GMURF_DIR, 0711) == -1) 
    if (errno != EEXIST) {
      printf("%s errno : %d\n", CANT_CREATE_TMP, errno);
      exit(1) ;
    }
  //if (wave_play("/home/ahes/Gmurf/MiniProg/Gmurf.wav") == -1)
  //print_gmurf_error("lors de l'ouverture d'un fichier WAV");
  if ((gmurfrc = gmurfrc_read(GMURFRC)) == NULL)
    {
      if (gmurfrc_write(gmurfrc, GMURFRC) == -1)
	{
#ifdef TEXT_MSG
	printf("erreur lors de l'ecriture du fichier de  configuration %s\n", GMURFRC);
#endif
	}
    }
  temp_gmurf = NULL;
  init_prog(&wav_path, &dsp_file, gmurfrc->first_dsp->sample_rate);
  gmurfrc = gmurfrc_defaults();
  dsp_file = gmurfrc->first_dsp->dsp;
  while (choix != 'q')
    {
      affiche_menu();
      choix = (char) getchar ();

      printf ("votre super choix est %c, bravo!\n", choix);
      if (choix == '1')
	{ 
	  copy(temp_gmurf, &select);
	}
      if (choix == '2')
	{
	  swap_chanels(temp_gmurf, &select);
	}
      if (choix == '3')
	{
	  earse(temp_gmurf, &select);
	}
      if (choix == '4')
	{
	  cut(temp_gmurf, &select);
	}
      if (choix == '5')
	{
	  translate(temp_gmurf, 1, 44100);
	}
      if (choix == 'c')
	{
	  char wav_path[80];
	  
	  printf("rate :%d\n", gmurfrc->first_dsp->sample_rate);
	  if (temp_gmurf != NULL)
	    {
	      close(temp_gmurf->file);
	      unlink(temp_gmurf->tempname);
	      free(temp_gmurf);
	    }
	  printf("\nEntrez le chemin du super fichier WAV a charger: ");
	  scanf("%s", wav_path);
	  if ((temp_gmurf = charge_sound_file(wav_path, "/tmp/Gmurf/temp_wav", gmurfrc->first_dsp->sample_rate)) != NULL)
	    is_wav = 0;
	  else
	    {
	      is_wav = 1;
	      print_gmurf_error("lors de la lecture d'un fichier sonore");
	    }
	  printf("\n");
	}
      if (choix == 'f')
	run_filter(temp_gmurf, filters, gmurfrc);
      if (choix == 'i')
	{
	  int	chanels2change;
	  
	  printf("quels canaux inverser\n 1: canal gauche\n 2: canal droit\n 3: les deux canaux\n");
	  scanf("%d", &chanels2change);
	  invert_chanels(temp_gmurf, chanels2change);
	} 
      if (choix == 'j')
	{
	  t_playmode playmode;
	  
	  playmode.chanels = 2;
	  printf("\n");
	  if (is_wav != 1)
	    play_temp_file_b(temp_gmurf->file, dsp_file->file, 0, 0, &playmode);
	  printf("\n");
	}
      if (choix == 'q') 
	break;
      else
	choix = (char) getchar ();
    }
  close_all(temp_gmurf, dsp_file);
  gmurfrc_destroy(gmurfrc);
  //if (wave_play("/home/ahes/Gmurf/MiniProg/Gmurf.wav") == -1)
  //print_gmurf_error("lors de l'ouverture d'un fichier WAV");
  return (1);
}
