/*
** GmurfProjectLoader.c
**
** Reads a Gmurf Mixer proprietary file. Untested
**
** sam-lo_p@epita.fr
*/


#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<fcntl.h>
#include<zlib.h>
#include"DebugLib.h"
#include"MixerTypes.h"
#include"MixerProtos.h"
#include"MixerCurvesProtos.h"
#include"MixerCurves.h"
#include"Constants.h"
#include"ConfigLib.h"
#include"Fct_Sys.h"

int remove_useless(data)
     char* data;

{
  char tmpstr[MAX_LINE_LEN];
  int i;
  int j;
  int len;

  i = 0;
  j = 0;

  len = strlen(data);

  while ((i <= len) && (data[i] != '#') && (data[i] != ';' )) // '#' and ';' are comments
    {
      if ((data[i] != ' ') && (data[i] != '\t'))
	{
	  tmpstr[j] = data[i];
	  j++;
	}
      
      i++;
    }
  tmpstr[i] = '\0';

  strcpy(data, tmpstr);
  return(0);
} // remove_spaces

char* find_temp_path(str)
     char* str;
{
  char* user;
  char* tmp_dir;

  user = getenv("USER") ;
  tmp_dir = (char *) malloc(strlen(user) + strlen(TMP_GMURF_DIR) + 2) ;
  strcpy(tmp_dir, TMP_GMURF_DIR);
  strcat(tmp_dir, "_") ;
  strcat(tmp_dir, user) ;

  strcpy(str, tmp_dir);

  free(tmp_dir);
  free(user);

  return(str);
}

int make_tempgmurf_file(src, dest)
     char* src;
     char* dest;
   
{
  static char tmptag[13] = "aaa.tempgmurf";
  gzFile      srcfd;
  int         dstfd;
  size_t      result;
  char* buf;	
  
  find_temp_path(dest);
  strcat(dest, "/");
  strcat(dest, tmptag);
  
  if (tmptag[2] <'z')
    {  
      tmptag[2]++;
    }
  else
    {
      if (tmptag[1] < 'z')
	{
	  tmptag[1]++;
	  tmptag[2] = 'a';
	}
      else
	{
	  tmptag[0]++;
	  tmptag[1] = 'a';
	  tmptag[2] = 'a';
	}
    }
  if ((srcfd = gzopen(src, "rb")) < 0)
    {
      warning("GmurfProjectLoader: make_tempgmurf_file: Couldn't open source file");
      return(-1);
    }
  
  if ((dstfd = open(dest, O_WRONLY | O_CREAT | O_TRUNC, 0644)) < 0)
    {
      warning("GmurfProjectLoader: make_tempgmurf_file: Couldn't open destination file");
      gzclose(srcfd);
      return(-2);
    }

  buf = (char*)malloc(CPY_BUF_SIZE);

  while ((result = gzread(srcfd, buf, CPY_BUF_SIZE)) > 0)
    write(dstfd, buf, result);

  gzclose(srcfd);
  close(dstfd);
  free(buf);

  return(0);

} // make_tempgmurf_file

int slashlen(char* str)
{
  int i;
  
  i = 0;
  while ((str[i] != '/') && (str[i] != '\0'))
    i++;
 i++;

  return(i);
}

int project_load(dir,file,mixer,gmurfrc)
     char*         dir;
     char*         file;
     t_main_mixer* mixer;
     t_gmurfrc*    gmurfrc;
     
{
  FILE*         fd;
  char*         str;
  char          cur_dir[MAX_LINE_LENGTH];
  char          tempname[MAX_LINE_LENGTH];
  char          tmpstr[MAX_LINE_LEN];
  t_mixer_elt*  mixer_elt;
  t_tempgmurf*  tempgmurf;
  t_mixer_info* mixerinfo;
  
  float    vol;
  float    pan;
  int      begin;
  int      x;
  int      y;
  int      nr_dsp;
  int      i;
  t_dsprc* dsprc;
  BOOL     frq_on, mute, rec;
  
  BOOL vol_ok, pan_ok, begin_ok, frq_ok, dsp_ok, mute_ok, rec_ok;
  
  mixer_elt = NULL;

  if ((strlen(file) <= GMURF_PRJ_EXT_LEN) || (strcmp(file + strlen(file)-GMURF_PRJ_EXT_LEN,GMURF_PROJECT_EXTENSION) != 0))
    {
      warning("GmurfProjectLoader.c: project_load: File extension does not match.");
      return(-2);
    }

  getcwd(cur_dir, MAX_LINE_LEN);

  if (chdir(dir) != 0)
    {
      warning("GmurfProjectLoader.c: project_load: Couldn't chdir. ");
      return(-1);
    }
  fd = fopen(file, "r");
  
  if (fd == NULL)
    {
      warning("GmurfProjectLoader.c: project_load: Can't open Gmurf Project.");
      return(-1);
    }
  
  str = (char*) malloc(sizeof(char)*MAX_LINE_LEN);
  
  while (!feof(fd)) 
    {
      fgets(str, MAX_LINE_LEN-1, fd);
      str[MAX_LINE_LEN-1] = '\0';
      if (strlen(str) == MAX_LINE_LEN-2)
	{
	  warning("GmurfProjectLoader.c: project_load: This file does not seem to be a Gmurf project.");
	  return(-2);
	}
      remove_useless(str);
      if (strcmp(str, ID_END) == 0)
	break;

      if (strcmp(str,ID_WAVE)==0) // new [wave] to be read
	{
	  str = fgets(str, MAX_LINE_LEN, fd);
	  str[strlen(str)-1]='\0';
	  
	  if (make_tempgmurf_file(str, tempname) < 0)
	    {
	      warning("GmurfProjectLoader: project_load: Critical error while "
		      "loading project.");
	      return(-3);
	    }
	  strncpy(tmpstr, str+slashlen(str), strlen(str)-slashlen(str)-GMURF_WAVE_EXT_LEN);
	  tmpstr[strlen(str)-slashlen(str)-GMURF_WAVE_EXT_LEN-4] = '\0';
	  tempgmurf = new_tempgmurf(tempname, tmpstr);
	  
	  vol_ok  = pan_ok = begin_ok = frq_ok = frq_on = mute = mute_ok = dsp_ok = rec = rec_ok = FALSE_;
	  vol    = DFT_VOL;
	  pan    = DFT_PAN;
	  begin  = DFT_START;
	  nr_dsp = 0;
	  tempgmurf->temp_size = give_file_size_by_name(tempname);
	}

      if (strcmp(str,ID_VOL)==0)
	{
	  vol       = (float) atoi(fgets(str, MAX_LINE_LEN, fd)) / DIV_VOLUME;
	  vol_ok    = TRUE_;
	}
      if (strcmp(str,ID_DSP)==0)
	{
	  nr_dsp    = atoi(fgets(str, MAX_LINE_LEN, fd)) ;
	  dsp_ok    = TRUE_;
	}
      if (strcmp(str,ID_PAN)==0)
	{
	  pan       = (float) atoi(fgets(str, MAX_LINE_LEN, fd)) / DIV_PANNING; 
	  pan_ok    = TRUE_;
	}
      if (strcmp(str, ID_START)==0)
	{
	  begin     = atoi(fgets(str, MAX_LINE_LEN, fd));
	  begin_ok  = TRUE_;
	}
      
      if (strcmp(str,ID_FRQON)==0)
	{
	  frq_on = TRUE_;
	  frq_ok = TRUE_;
	}

      if (strcmp(str, ID_MUTE)==0)
	{
	  mute = TRUE_;
	  mute_ok = TRUE_;
	}

      if (strcmp(str,ID_REC)==0)
	{
	  rec =    TRUE_;
	  rec_ok = TRUE_;
	}
     
      if (strcmp(str, ID_ENDWAVE)==0)
	{
	  if (!vol_ok)
	    parse_warning("Volume",tempgmurf->tempname);
	  
	  //	  if (!frq_ok)
	  // parse_warning("Frequency flag",tempgmurf->tempname);
	  
	  if (!pan_ok)
	    parse_warning("Panning",tempgmurf->tempname);
	  
	  if (!begin_ok)
	    parse_warning("Start",tempgmurf->tempname);
  
	  if (!dsp_ok)
	    parse_warning("Dsp number",tempgmurf->tempname);

	  mixerinfo = new_mixer_info(vol,frq_on,pan,begin);
	  dsprc = gmurfrc->first_dsp;
	  
	  for (i=0; i<nr_dsp; i++)
	    if (dsprc->next_dsp != NULL) 
	      dsprc = dsprc->next_dsp;
	    else
	      warning("GmurfProjectLoader.c: project_load: Not enough DSPs to respect "
		      "the project's data. This may not be a problem.");
	  
	  mixer_elt = add_mixer_elt(mixer, dsprc->dsp, tempgmurf, 
				    mixerinfo);
	  mixer_elt->mute = mute;
	  mixer_elt->rec_or_play = rec;

	}
	 
      if ((strcmp(str, ID_VOLPOINT)==0) && (mixer_elt != NULL)) // new vol point
	{
	  x = atoi(fgets(str, MAX_LINE_LEN, fd));
	  y = atoi(fgets(str, MAX_LINE_LEN, fd));
	  add_vol_point(mixer_elt,x,y);
	}
      
      if ((strcmp(str, ID_PANPOINT)==0) && (mixer_elt != NULL)) // new pan point
	{
	  x = atoi(fgets(str, MAX_LINE_LEN, fd));
	  y = atoi(fgets(str, MAX_LINE_LEN, fd));
	  add_pan_point(mixer_elt,x,y);
	}
      
      if ((strcmp(str, ID_FRQPOINT)==0) && (mixer_elt != NULL)) // new frq point
	{
	  x = atoi(fgets(str, MAX_LINE_LEN, fd));
	  y = atoi(fgets(str, MAX_LINE_LEN, fd));
	  add_frq_point(mixer_elt,x,y);
	}
      
    }
  free(str);
  fclose(fd);
  if (chdir(cur_dir) != 0)
    {
      warning("GmurfProjectLoader.c: project_load: Can't chdir to the original"
	      " directory");
      return(-4);
    }
  return (0);
} // parse

