#include "waveditor.h"

static void	waveditor_class_init		(WaveditorClass		*klass);
static void	waveditor_init			(Waveditor 		*wve);
static void	channel_left_button		(GtkWidget		*widget, 
						 GdkEventButton 	*event, 
						 Waveditor		*wve) ;
static void	channel_left_motion		(GtkWidget		*widget, 
						 GdkEventMotion 	*event, 
						 Waveditor		*wve) ;
static int	channel_left_expose		(GtkWidget		*widget, 
						 GdkEventExpose 	*event, 
						 Waveditor		*wve) ;
static void	channel_left_configure		(GtkWidget		*widget, 
						 GdkEventConfigure 	*event, 
						 Waveditor		*wve) ;
static void	channel_right_button		(GtkWidget		*widget, 
						 GdkEventButton 	*event, 
						 Waveditor		*wve) ;
static void	channel_right_motion		(GtkWidget		*widget, 
						 GdkEventMotion 	*event, 
						 Waveditor		*wve) ;
static int	channel_right_expose		(GtkWidget		*widget, 
						 GdkEventExpose 	*event, 
						 Waveditor		*wve) ;
static void	channel_right_configure		(GtkWidget		*widget, 
						 GdkEventConfigure 	*event, 
						 Waveditor		*wve) ;

GtkWidget*	waveditor_new			(void);

enum {
	LEFT,
	RIGHT
} ;

enum {
	CHANNEL_LEFT_BUTTON,
	CHANNEL_RIGHT_BUTTON,
	CHANNEL_LEFT_MOTION,
	CHANNEL_RIGHT_MOTION,
	CHANNEL_LEFT_EXPOSE,
	CHANNEL_RIGHT_EXPOSE,
	CHANNEL_LEFT_CONFIGURE,
	CHANNEL_RIGHT_CONFIGURE,
	LAST_SIGNAL
};

gpointer wve_signal_func[LAST_SIGNAL] = {
  channel_left_button, channel_right_button,
  channel_left_motion, channel_right_motion,
  channel_left_expose, channel_right_expose,
  channel_left_configure, channel_right_configure
} ;

char *wve_signal_name[LAST_SIGNAL] = {
  "channel_left_button", "channel_right_button",
  "channel_left_motion", "channel_right_motion",
  "channel_left_expose", "channel_right_expose",
  "channel_left_configure", "channel_right_configure"
} ;

static gint waveditor_signals[LAST_SIGNAL] = { 0, 0, 0, 0, 0, 0, 0, 0 };


guint waveditor_get_type(void)
{
  static guint wve_type = 0;

  if (!wve_type)
    {
      GtkTypeInfo wve_info =
      {
	"Waveditor",
	sizeof (Waveditor),
	sizeof (WaveditorClass),
	(GtkClassInitFunc) waveditor_class_init,
	(GtkObjectInitFunc) waveditor_init,
        (GtkArgSetFunc) NULL
//        (GtkArgGetFunc) NULL
      };

      wve_type = gtk_type_unique (gtk_vbox_get_type (), &wve_info);
    }

  return wve_type;
}

static void	waveditor_class_init (class)
WaveditorClass	*class ;
{
  GtkObjectClass	*object_class;
  int			i ;
  
  object_class = (GtkObjectClass*) class;  
  for (i = 0; i < LAST_SIGNAL; i++){
    waveditor_signals[i] = 
      gtk_signal_new (wve_signal_name[i],
		      GTK_RUN_FIRST,
		      object_class->type,
		      GTK_SIGNAL_OFFSET (WaveditorClass, 
					 waveditor),
		      gtk_marshal_NONE__POINTER,
		      GTK_TYPE_NONE, 1,
		      GTK_TYPE_POINTER);
  }
  gtk_object_class_add_signals (object_class, 
				waveditor_signals, LAST_SIGNAL);
  class->waveditor = NULL;
}

GtkWidget *waveditor_new(void)
{
  return GTK_WIDGET ( gtk_type_new (waveditor_get_type ()));
}


static void	waveditor_init(wve)
Waveditor	*wve ;
{
  int		i;
  GtkWidget	*frame ;  
  char		*frame_name[2] = {
    "Channel Left", "Channel Right" 
  } ;
  GtkAdjustment	*adjustment ;  
  GtkWidget	*hbox ;
  GtkWidget	*label ;

  wve->there_is_a_selection = FALSE ;
  wve->selection.x = 0 ;
  wve->selection.y = 0 ;
  

  adjustment = (GtkAdjustment *) gtk_adjustment_new(0, 0, IWIDTH, 5, 1, IWIDTH);
  wve->hscrollbar = gtk_hscrollbar_new(adjustment) ;
  gtk_widget_show(wve->hscrollbar) ;
  gtk_box_pack_start(GTK_BOX (wve), wve->hscrollbar, FALSE, FALSE, 0) ;

  /**************** Position Hbox **************/
  hbox = gtk_hbox_new(FALSE, 0) ;
  gtk_widget_show(hbox) ;
  label = gtk_label_new("Position : in Sound Unit") ;
  gtk_widget_show(label) ;
  gtk_box_pack_start(GTK_BOX (hbox), label, FALSE, FALSE, 2 ) ;
  
  wve->SU_label = gtk_label_new("0") ;
  gtk_widget_show(wve->SU_label) ;
  gtk_box_pack_start(GTK_BOX (hbox), wve->SU_label, TRUE, FALSE, 0 ) ;

  label = gtk_label_new("in Miliseconds : ") ;
  gtk_widget_show(label) ;
  gtk_box_pack_start(GTK_BOX (hbox), label, FALSE, FALSE, 2 ) ;

  wve->Sec_label = gtk_label_new("0") ;
  gtk_widget_show(wve->Sec_label) ;
  gtk_box_pack_start(GTK_BOX (hbox), wve->Sec_label, TRUE, FALSE, 0 ) ;

//  wve->hruler = gtk_hruler_new() ;
//  gtk_widget_show(wve->hruler) ;
  gtk_box_pack_start(GTK_BOX (wve), hbox, FALSE, FALSE, 0) ;

  for (i = 0; i<CHANNELS; i++){
    wve->pixmap[i] = NULL ;  
    
/* The drawing area */
    wve->DrawingArea[i] = gtk_drawing_area_new() ;
    gtk_drawing_area_size(GTK_DRAWING_AREA(wve->DrawingArea[i]), 
			  IWIDTH, IHEIGHT);
    gtk_signal_connect(GTK_OBJECT (wve->DrawingArea[i]), "button_press_event",
		       GTK_SIGNAL_FUNC (wve_signal_func[i]), wve);
    gtk_signal_connect(GTK_OBJECT (wve->DrawingArea[i]), "motion_notify_event",
		       (GtkSignalFunc) wve_signal_func[i + CHANNEL_LEFT_MOTION], wve);
    gtk_signal_connect(GTK_OBJECT (wve->DrawingArea[i]), "expose_event",
		       (GtkSignalFunc) wve_signal_func[i + CHANNEL_LEFT_EXPOSE], wve);
    gtk_signal_connect(GTK_OBJECT(wve->DrawingArea[i]),"configure_event",
    		       (GtkSignalFunc) wve_signal_func[i + CHANNEL_LEFT_CONFIGURE], wve);
    gtk_widget_set_events(wve->DrawingArea[i], GDK_EXPOSURE_MASK
			  | GDK_LEAVE_NOTIFY_MASK
			  | GDK_BUTTON_PRESS_MASK
			  | GDK_BUTTON_RELEASE_MASK
			  | GDK_POINTER_MOTION_MASK
	 		  | GDK_POINTER_MOTION_HINT_MASK);
    gtk_widget_show(wve->DrawingArea[i]) ;
  
    /*  gtk_signal_connect_object(GTK_OBJECT (wve->DrawingArea[i]), 
			      "motion_notify_event",
			      (GtkSignalFunc) EVENT_METHOD (wve->hruler, 
							    motion_notify_event),
			      GTK_OBJECT (wve->hruler));
    */
  
    /* Put a frame and the drawing area in */
    frame = gtk_frame_new(frame_name[i]) ;
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN) ;
    gtk_container_add(GTK_CONTAINER (frame), wve->DrawingArea[i]) ;    
    gtk_container_border_width(GTK_CONTAINER (frame), 2) ;
    gtk_widget_set_usize(frame, IWIDTH, IHEIGHT) ;
    gtk_box_pack_start(GTK_BOX (wve), frame, TRUE, FALSE, 0);
    gtk_widget_show(frame) ;
  }
}

 
static void	channel_left_button	(widget, 
					 event, 
					 wve)
GtkWidget	*widget ;
GdkEventButton	*event ;  
Waveditor	*wve ;
{
    gtk_signal_emit (GTK_OBJECT (wve), 
		     waveditor_signals[CHANNEL_LEFT_BUTTON],
		     (gpointer) event);
}

static void	channel_right_button	(widget, 
					 event, 
					 wve)
GtkWidget	*widget ;
GdkEventButton	*event ;  
Waveditor	*wve ;
{
  gtk_signal_emit (GTK_OBJECT (wve), 
		   waveditor_signals[CHANNEL_RIGHT_BUTTON],
		   (gpointer) event);
}

 
static void	channel_left_motion	(widget, 
					 event, 
					 wve)
GtkWidget	*widget ;
GdkEventMotion	*event ;  
Waveditor	*wve ;
{
  gtk_signal_emit (GTK_OBJECT (wve), 
		   waveditor_signals[CHANNEL_LEFT_MOTION],
		   (gpointer) event);
}

static void	channel_right_motion	(widget, 
					 event, 
					 wve)
GtkWidget	*widget ;
GdkEventMotion	*event ;  
Waveditor	*wve ;
{
  gtk_signal_emit (GTK_OBJECT (wve), 
		   waveditor_signals[CHANNEL_RIGHT_MOTION],
		   (gpointer) event);
}

static int	channel_left_expose	(widget, 
					 event, 
					 wve)
GtkWidget	*widget ;
GdkEventExpose	*event ;  
Waveditor	*wve ;
{
  gtk_signal_emit (GTK_OBJECT (wve), 
  		   waveditor_signals[CHANNEL_LEFT_EXPOSE],
		   (gpointer) event);
  return FALSE ;
}

static int	channel_right_expose	(widget, 
					 event, 
					 wve)
GtkWidget	*widget ;
GdkEventExpose	*event ;  
Waveditor	*wve ;
{ 
  gtk_signal_emit (GTK_OBJECT (wve), 
		   waveditor_signals[CHANNEL_RIGHT_EXPOSE],
		   (gpointer) event);
  return FALSE ;
}

 
static void	channel_left_configure	(widget, 
					 event, 
					 wve)
GtkWidget		*widget ;
GdkEventConfigure	*event ;  
Waveditor		*wve ;
{
    gtk_signal_emit (GTK_OBJECT (wve), 
		     waveditor_signals[CHANNEL_LEFT_CONFIGURE],
		     (gpointer) event);
}

static void	channel_right_configure	 (widget, 
					  event, 
					  wve)
GtkWidget		*widget ;
GdkEventConfigure	*event ;  
Waveditor		*wve ;
{
  gtk_signal_emit (GTK_OBJECT (wve), 
		   waveditor_signals[CHANNEL_RIGHT_CONFIGURE],
		   (gpointer) event);
}
















